@@ -95,8 +95,7 @@ class ValDrop(ColumnsBasedPipelineStage):
         self._values_str = _list_str(self._values)
         super_kwargs = {
             'columns': columns,
-            'desc_temp': 'Drop values {} in columns {{}}'.format(
-                self._values_str),
+            'desc_temp': f'Drop values {self._values_str} in columns {{}}',
         }
         super_kwargs.update(**kwargs)
         super_kwargs['none_columns'] = 'all'
@@ -109,7 +108,7 @@ class ValDrop(ColumnsBasedPipelineStage):
         for col in columns_to_check:
             inter_df = inter_df[~inter_df[col].isin(self._values)]
         if verbose:
-            print("{} rows dropped.".format(before_count - len(inter_df)))
+            print(f"{before_count - len(inter_df)} rows dropped.")
         return inter_df
 
 
@@ -149,8 +148,7 @@ class ValKeep(ColumnsBasedPipelineStage):
         self._values_str = _list_str(self._values)
         super_kwargs = {
             'columns': columns,
-            'desc_temp': 'Keep values {} in columns {{}}'.format(
-                self._values_str),
+            'desc_temp': f'Keep values {self._values_str} in columns {{}}',
         }
         super_kwargs.update(**kwargs)
         super_kwargs['none_columns'] = 'all'
@@ -163,7 +161,7 @@ class ValKeep(ColumnsBasedPipelineStage):
         for col in columns_to_check:
             inter_df = inter_df[inter_df[col].isin(self._values)]
         if verbose:
-            print("{} rows dropped.".format(before_count - len(inter_df)))
+            print(f"{before_count - len(inter_df)} rows dropped.")
         return inter_df
 
 
@@ -172,7 +170,7 @@ class ColRename(PdPipelineStage):
 
     Parameters
     ----------
-    rename_map : dict
+    rename_mapper : dict-like or function
         Maps old column names to new ones.
 
     Example
@@ -188,22 +186,45 @@ class ColRename(PdPipelineStage):
     _DEF_COLDRENAME_EXC_MSG = ("ColRename stage failed because not all columns"
                                " {} were found in input dataframe.")
 
-    def __init__(self, rename_map, **kwargs):
-        self._rename_map = rename_map
-        columns_str = _list_str(list(rename_map.keys()))
-        suffix = 's' if len(rename_map) > 1 else ''
+    def __init__(self, rename_mapper, **kwargs):
+        self._rename_mapper = rename_mapper
+        try:
+            columns_str = _list_str(list(rename_mapper.keys()))
+            mapper_repr = str(rename_mapper)
+            keys_set = set(self._rename_mapper.keys())
+
+            def _tprec(df):
+                return keys_set.issubset(df.columns)
+        except AttributeError:
+            mapper_repr = rename_mapper.__name__
+            doc = rename_mapper.__doc__
+            if doc is None:
+                columns_str = f"by func {rename_mapper.__name__}"
+            else:
+                columns_str = (
+                    f"by func {rename_mapper.__name__} with "
+                    f"doc: {rename_mapper.__doc__}"
+                )
+
+            def _tprec(df):
+                return True
+        try:
+            suffix = 's' if len(rename_mapper) > 1 else ''
+        except TypeError:
+            suffix = 's'
+        self._tprec = _tprec
         super_kwargs = {
             'exmsg': ColRename._DEF_COLDRENAME_EXC_MSG.format(columns_str),
-            'desc': "Rename column{} with {}".format(suffix, self._rename_map)
+            'desc': f"Rename column{suffix} with {mapper_repr}",
         }
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
     def _prec(self, df):
-        return set(self._rename_map.keys()).issubset(df.columns)
+        return self._tprec(df)
 
     def _transform(self, df, verbose):
-        return df.rename(columns=self._rename_map)
+        return df.rename(columns=self._rename_mapper)
 
 
 class DropNa(PdPipelineStage):
@@ -243,13 +264,62 @@ class DropNa(PdPipelineStage):
         ncols_before = len(df.columns)
         inter_df = df.dropna(**self.dropna_kwargs)
         if verbose:
-            print("{} rows, {} columns dropeed".format(
-                before_count - len(inter_df),
-                ncols_before - len(inter_df.columns),
-            ))
+            print(
+                f"{before_count - len(inter_df)} rows, "
+                f"{ncols_before - len(inter_df.columns)} columns dropeed"
+            )
         return inter_df
 
 
+class SetIndex(PdPipelineStage):
+    """A pipeline stage that set existing columns as index.
+
+    Supports all parameter supported by pandas.set_index function except for
+    `inplace`.
+
+    Example
+    -------
+        >> import pandas as pd; import pdpipe as pdp;
+        >> df = pd.DataFrame([[1,4],[3, 11]], [1,2], ['a','b'])
+        >> pdp.SetIndex('a').apply(df)
+            b
+        a
+        1   4
+        3  11
+    """
+
+    _DEF_SETIDX_EXC_MSG = "SetIndex stage failed."
+    _DEF_SETIDX_APP_MSG = "Setting indexes..."
+    _SETINDEX_KWARGS = ['drop', 'append', 'verify_integrity']
+
+    def __init__(self, keys, **kwargs):
+        common = set(kwargs.keys()).intersection(SetIndex._SETINDEX_KWARGS)
+        self.setindex_kwargs = {key: kwargs.pop(key) for key in common}
+        self.keys = keys
+        if isinstance(keys, str):
+            def _tprec(df):
+                return keys in df.columns
+        elif hasattr(keys, '__iter__'):
+            def _tprec(df):
+                return all([k in df.columns for k in keys])
+        else:
+            def _tprec(df):
+                return True
+        self._tprec = _tprec
+        super_kwargs = {
+            'exmsg': SetIndex._DEF_SETIDX_EXC_MSG,
+            'desc': "Set indexes."
+        }
+        super_kwargs.update(**kwargs)
+        super().__init__(**super_kwargs)
+
+    def _prec(self, df):
+        return self._tprec(df)
+
+    def _transform(self, df, verbose):
+        return df.set_index(keys=self.keys, **self.setindex_kwargs)
+
+
 class FreqDrop(PdPipelineStage):
     """A pipeline stage that drops rows by value frequency.
 
@@ -295,7 +365,7 @@ class FreqDrop(PdPipelineStage):
         to_drop = valcount[valcount < self._threshold].index
         inter_df = inter_df[~inter_df[self._column].isin(to_drop)]
         if verbose:
-            print("{} rows dropped.".format(before_count - len(inter_df)))
+            print(f"{before_count - len(inter_df)} rows dropped.")
         return inter_df
 
 
@@ -326,7 +396,7 @@ class ColReorder(PdPipelineStage):
         self._pos_to_col = reverse_dict_partial(positions)
         super_kwargs = {
             'exmsg': ColReorder._DEF_ORD_EXC_MSG.format(self._col_to_pos),
-            'desc': "Reorder columns by {}".format(self._col_to_pos),
+            'desc': f"Reorder columns by {self._col_to_pos}",
         }
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
@@ -347,8 +417,7 @@ class ColReorder(PdPipelineStage):
                     new_columns.append(non_map_cols.popleft())
             return df[new_columns]
         except (IndexError):
-            raise ValueError("Bad positions mapping given: {}".format(
-                new_columns))
+            raise ValueError(f"Bad positions mapping given: {new_columns}")
 
 
 class RowDrop(ColumnsBasedPipelineStage):
@@ -473,7 +542,7 @@ class RowDrop(ColumnsBasedPipelineStage):
         drop_index = ~subdf.apply(self._row_cond, axis=1)
         inter_df = df[drop_index]
         if verbose:
-            print("{} rows dropped.".format(before_count - len(inter_df)))
+            print(f"{before_count - len(inter_df)} rows dropped.")
         return inter_df
 
 
@@ -502,10 +571,14 @@ class Schematize(PdPipelineStage):
     def __init__(self, columns, **kwargs):
         self._columns = _interpret_columns_param(columns)
         self._columns_str = _list_str(self._columns)
-        desc = "Transform input dataframes to the following schema: {}".format(
-            self._columns_str)
-        exmsg = "Not all required columns {} found in input dataframe!".format(
-            self._columns_str)
+        desc = (
+            f"Transform input dataframes to the following schema: "
+            f"{self._columns_str}"
+        )
+        exmsg = (
+            f"Not all required columns {self._columns_str} "
+            f"found in input dataframe!"
+        )
         super_kwargs = {
             'exmsg': exmsg,
             'desc': desc,
@@ -557,5 +630,5 @@ class DropDuplicates(ColumnsBasedPipelineStage):
         columns = self._get_columns(df, fit=fit)
         inter_df = df.drop_duplicates(subset=columns)
         if verbose:
-            print("{} rows dropped.".format(len(df) - len(inter_df)))
+            print(f"{len(df) - len(inter_df)} rows dropped.")
         return inter_df
