@@ -101,6 +101,7 @@ from .cq import is_fittable_column_qualifier, AllColumns
 from .exceptions import (
     FailedPreconditionError,
     UnfittedPipelineStageError,
+    PipelineApplicationError
 )
 
 
@@ -172,24 +173,33 @@ class PdPipelineStage(abc.ABC):
         dataframes, which will be used to determine whether this stage should
         be skipped for input dataframes. See pdp.cond for more information on
         specialised Condition objects.
+    name : str, default ''
+        The name of this stage.
+        Pipelines can be sliced by this name.
     """
 
     _DEF_EXC_MSG = 'Precondition failed in stage {}!'
     _DEF_DESCRIPTION = 'A pipeline stage.'
-    _INIT_KWARGS = ['exraise', 'exmsg', 'desc', 'prec', 'skip']
+    _INIT_KWARGS = ['exraise', 'exmsg', 'desc', 'prec', 'skip', 'name']
 
     def __init__(self, exraise=True, exmsg=None, desc=None, prec=None,
-                 skip=None):
+                 skip=None, name=''):
+        if not isinstance(name, str):
+            raise ValueError(
+                f"'name' must be a str, not {type(name).__name__}."
+            )
         if desc is None:
             desc = PdPipelineStage._DEF_DESCRIPTION
         if exmsg is None:
             exmsg = PdPipelineStage._DEF_EXC_MSG.format(desc)
+
         self._exraise = exraise
         self._exmsg = exmsg
         self._desc = desc
         self._prec_arg = prec
         self._skip = skip
-        self._appmsg = '{}..'.format(desc)
+        self._appmsg = f"{name + ': ' if name else ''}{desc}"
+        self._name = name
         self.is_fitted = False
 
     @classmethod
@@ -388,7 +398,7 @@ class PdPipelineStage(abc.ABC):
         return NotImplemented
 
     def __str__(self):
-        return "PdPipelineStage: {}".format(self._desc)
+        return f"PdPipelineStage: {self._desc}"
 
     def __repr__(self):
         return self.__str__()
@@ -651,6 +661,17 @@ class PdPipeline(PdPipelineStage, collections.abc.Sequence):
     def __getitem__(self, index):
         if isinstance(index, slice):
             return PdPipeline(self._stages[index])
+
+        if isinstance(index, list) and all(isinstance(x, str) for x in index):
+            stages = [stage for stage in self._stages if stage._name in index]
+            return PdPipeline(stages)
+
+        if isinstance(index, str):
+            stages = [stage for stage in self._stages if stage._name == index]
+            if len(stages) == 0:
+                raise ValueError(f"'{index}' is not exist.")
+            return stages[0]
+
         return self._stages[index]
 
     # implementing a collections.abc.Sequence abstract method
@@ -674,16 +695,21 @@ class PdPipeline(PdPipelineStage, collections.abc.Sequence):
         inter_x = X
         times = []
         prev = time.time()
-        for stage in self._stages:
-            inter_x = stage.fit_transform(
-                X=inter_x,
-                y=None,
-                exraise=exraise,
-                verbose=verbose,
-            )
-            now = time.time()
-            times.append(now - prev)
-            prev = now
+        for i, stage in enumerate(self._stages):
+            try:
+                inter_x = stage.fit_transform(
+                    X=inter_x,
+                    y=None,
+                    exraise=exraise,
+                    verbose=verbose,
+                )
+                now = time.time()
+                times.append(now - prev)
+                prev = now
+            except Exception:
+                raise PipelineApplicationError(
+                    f"Exception raised in stage [ {i}] {stage}"
+                )
         self.is_fitted = True
         print("\nPipeline total application time: {:.3f}s.\n Details:".format(
             sum(times)))
@@ -723,13 +749,18 @@ class PdPipeline(PdPipelineStage, collections.abc.Sequence):
             return self.__timed_fit_transform(
                 X=X, y=y, exraise=exraise, verbose=verbose)
         inter_x = X
-        for stage in self._stages:
-            inter_x = stage.fit_transform(
-                X=inter_x,
-                y=None,
-                exraise=exraise,
-                verbose=verbose,
-            )
+        for i, stage in enumerate(self._stages):
+            try:
+                inter_x = stage.fit_transform(
+                    X=inter_x,
+                    y=None,
+                    exraise=exraise,
+                    verbose=verbose,
+                )
+            except Exception:
+                raise PipelineApplicationError(
+                    f"Exception raised in stage [ {i}] {stage}"
+                )
         self.is_fitted = True
         return inter_x
 
@@ -775,16 +806,21 @@ class PdPipeline(PdPipelineStage, collections.abc.Sequence):
         inter_x = X
         times = []
         prev = time.time()
-        for stage in self._stages:
-            inter_x = stage.transform(
-                X=inter_x,
-                y=None,
-                exraise=exraise,
-                verbose=verbose,
-            )
-            now = time.time()
-            times.append(now - prev)
-            prev = now
+        for i, stage in enumerate(self._stages):
+            try:
+                inter_x = stage.transform(
+                    X=inter_x,
+                    y=None,
+                    exraise=exraise,
+                    verbose=verbose,
+                )
+                now = time.time()
+                times.append(now - prev)
+                prev = now
+            except Exception:
+                raise PipelineApplicationError(
+                    f"Exception raised in stage [ {i}] {stage}"
+                )
         self.is_fitted = True
         print("\nPipeline total application time: {:.3f}s.\n Details:".format(
             sum(times)))
@@ -832,13 +868,18 @@ class PdPipeline(PdPipelineStage, collections.abc.Sequence):
             return self.__timed_transform(
                 X=X, y=y, exraise=exraise, verbose=verbose)
         inter_df = X
-        for stage in self._stages:
-            inter_df = stage.transform(
-                X=inter_df,
-                y=None,
-                exraise=exraise,
-                verbose=verbose,
-            )
+        for i, stage in enumerate(self._stages):
+            try:
+                inter_df = stage.transform(
+                    X=inter_df,
+                    y=None,
+                    exraise=exraise,
+                    verbose=verbose,
+                )
+            except Exception:
+                raise PipelineApplicationError(
+                    f"Exception raised in stage [ {i}] {stage}"
+                )
         return inter_df
 
     __call__ = apply
